@@ -1,4 +1,4 @@
-import numpy,pandas,os,sys,time,scipy.io,scipy.sparse,scipy.stats
+import numpy,pandas,os,sys,time,scipy.io,scipy.sparse,scipy.stats,subprocess
 #import numba
 import matplotlib
 matplotlib.use('Agg')
@@ -145,8 +145,8 @@ def get_tss_peaks(project, distal=20000):
                                 columns=['chrom', 'center'])
     tss_df = pandas.read_csv(project+'/peak/genes_tss_region.csv', sep='\t', index_col=0)
     for gene in tss_df.index.values:
-        chrom, tsses = tss_df.ix[gene, 'chrom'], tss_df.ix[gene, 'tss']
-        tsses = map(int, tsses.split(','))
+        chrom, tsses = tss_df.loc[gene, 'chrom'], tss_df.loc[gene, 'tss']
+        tsses = list(map(int, tsses.split(',')))
         chr_peaks = peaks_df.loc[peaks_df['chrom']==chrom]
         proxim_peaks, distal_peaks = [], []
         for tss in tsses:
@@ -159,9 +159,9 @@ def get_tss_peaks(project, distal=20000):
         if len(proxim_peaks)==0: proxim_peaks = ['NONE']
         if len(distal_peaks)==0: distal_peaks = ['NONE']
         proxim_peaks = ';'.join(proxim_peaks)
-        tss_df.ix[gene, 'proximal'] = proxim_peaks
+        tss_df.loc[gene, 'proximal'] = proxim_peaks
         distal_peaks = ';'.join(distal_peaks)
-        tss_df.ix[gene, 'distal'] = distal_peaks
+        tss_df.loc[gene, 'distal'] = distal_peaks
     tss_df.to_csv(project+'/peak/genes_TSS_peaks.csv', sep='\t')
     return
 #
@@ -199,36 +199,20 @@ def get_score_from_peaks(project):
     return
 #
 #
-#
-#@numba.jit()
-def nearby_genes(project, genome_gtf, distal=20000):
-    """
-    project:     Path to the project folder.
-    genome_gtf:  hg19_RefSeq_genes.gtf or mm10_RefSeq_genes.gtf in reference folder
-    distal:      Genome distance for distal region, default=20000.
-    """
-    peaks = [[x.split()[0], (int(x.split()[1])+int(x.split()[2]))/2]
-             for x in open(project+'/peak/top_filtered_peaks.bed').readlines()]
-    peaks_df = pandas.DataFrame(peaks, index=['peak'+str(x) for x in numpy.arange(0,len(peaks))],
-                                columns=['chrom', 'center'])
-    get_tss_region(project, genome_gtf)
-    tss_df = pandas.read_csv(project+'/peak/genes_tss_region.csv', sep='\t', index_col=0)
-    for ipeak,peak in enumerate(peaks_df.index.values):
-        chrom, center = peaks_df.loc[peak, 'chrom'], peaks_df.loc[peak, 'center']
-        sub_tss_df = tss_df.loc[tss_df['chrom']==chrom]
-        close_genes = []
-        for gene in sub_tss_df.index.values:
-            positions = numpy.array(list(map(int, sub_tss_df.loc[gene, 'tss'].split(','))))
-            close_index = numpy.where(abs(positions-center)<=distal)[0]
-            if len(close_index)>0:
-                close_genes.append(gene)
-        if len(close_genes)>0:
-            peaks_df.loc[peak, 'nearby_genes'] = ';'.join(close_genes)
-        else:
-            peaks_df.loc[peak, 'nearby_genes'] = 'NONE'
-        if ipeak%1000==0: print(ipeak, close_genes)
-    peaks_df.to_csv(project+'/peak/peaks_nearby_genes.csv', sep='\t')
-    print('nearby_genes output files:')
+def get_nearby_genes(project):
+    gene_peaks = pandas.read_csv(project+'/peak/genes_TSS_peaks.csv', sep='\t', index_col=0)
+    peak_genes = {}
+    for gg in gene_peaks.index.values:
+        gg_peaks = gene_peaks.loc[gg,'proximal'].split(';') + gene_peaks.loc[gg,'distal'].split(';')
+        gg_peaks = list(set(gg_peaks) - set(['NONE']))
+        for pp in gg_peaks:
+            if 'peak'+pp not in peak_genes.keys():
+                peak_genes['peak'+pp] = gg
+            else:
+                peak_genes['peak'+pp] = peak_genes['peak'+pp] + ';' + gg
+    peak_genes_df = pandas.DataFrame.from_dict(peak_genes, orient='index', columns=['nearby_genes'])
+    peak_genes_df.to_csv(project+'/peak/peaks_nearby_genes.csv', sep='\t')
+    print('get_nearby_genes output files:')
     print(project+'/peak/peaks_nearby_genes.csv')
     return
 #
@@ -248,102 +232,6 @@ def gene_score(project, genome_gtf='', distal=20000):
     return
 #
 #
-#### generate gene expression
-#
-#
-def annotate_peak(project, genome='hg19'):
-    peaks_bed = project + '/peak/top_filtered_peaks.bed'
-    annotated = project + '/peak/top_annotated_peaks.bed'
-    os.popen('annotatePeaks.pl '+peaks_bed+'  '+genome+' -strand both -size given > '+annotated)
-    return
-#
-#
-def annotate_accessons(project, width=1e6, pvalue=0.01):
-    peak_bed = project + '/peak/top_filtered_peaks.bed'
-    accesson_csv = project + '/matrix/Accesson_peaks.csv'
-    annotated_bed = project + '/peak/top_annotated_peaks.bed'
-    peak_list = [[x.split()[0], (int(x.split()[1])+int(x.split()[2]))//2] for x in open(peak_bed).readlines()]
-    peak_index = ['peak'+str(x+1) for x in range(0, len(peak_list))]
-    peak_df = pandas.DataFrame(peak_list, index=peak_index, columns=['chromosome', 'base'])
-    annotated_df = pandas.read_csv(annotated_bed, sep='\t', index_col=0,
-                                   engine='c', na_filter=False, low_memory=False)
-    annotated_df.index = ['peak'+str(x).split('-')[-1] for x in annotated_df.index.values]
-    accesson_df = pandas.read_csv(accesson_csv, sep='\t', index_col=0,
-                                  engine='c', na_filter=False, low_memory=False)
-    accessons = list(set(accesson_df['group'].values))
-    accessons.sort()
-    accesson_annotate = pandas.DataFrame(columns=['genes', '-log10(P-value)'])
-    gene_annotate = pandas.DataFrame(columns=['accessons', '-log10(P-value)'])
-    for acc in accessons:
-        accPeak_df = peak_df.loc[accesson_df.loc[accesson_df['group']==acc].index]
-        genes = {}
-        for peak in accPeak_df.index.values:
-            chrom, base = accPeak_df.loc[peak, 'chromosome'], accPeak_df.loc[peak, 'base']
-            base_up, base_down = base - width, base + width
-            sameChrom_in_acc = accPeak_df.loc[accPeak_df['chromosome']==chrom]
-            sameChrom_overall = peak_df.loc[peak_df['chromosome']==chrom]
-            sameRegion_in_acc = numpy.where(abs(sameChrom_in_acc['base']-base)<=width)[0]
-            sameRegion_overall = numpy.where(abs(sameChrom_overall['base']-base)<=width)[0]
-            matrix = numpy.array([[len(sameRegion_in_acc), len(accPeak_df)-len(sameRegion_in_acc)],
-                                  [len(sameRegion_overall), len(peak_df)-len(sameRegion_overall)]])
-            odd, p_value = scipy.stats.fisher_exact(matrix)
-            if p_value<=pvalue:
-                gene_symbol = annotated_df.loc[peak, 'Gene Name']
-                log10_P = -numpy.log10(p_value)
-                if gene_symbol not in genes.keys():
-                    genes[gene_symbol] = log10_P
-                elif genes[gene_symbol] < log10_P:
-                    genes[gene_symbol] = log10_P
-        for gene in genes.keys():
-            if gene not in gene_annotate.index.values:
-                gene_annotate.loc[gene, 'accessons'] = str(acc)
-                gene_annotate.loc[gene, '-log10(P-value)'] = str(genes[gene])
-            else:
-                gene_annotate.loc[gene, 'accessons'] += ';' + str(acc)
-                gene_annotate.loc[gene, '-log10(P-value)'] += ';' + str(genes[gene])
-        accesson_annotate.loc[acc, 'genes'] = ';'.join(genes.keys())
-        accesson_annotate.loc[acc, '-log10(P-value)'] = ';'.join(list(map(str, genes.values())))
-    accesson_annotate.to_csv(project+'/matrix/Accesson_annotated.csv', sep='\t')
-    gene_annotate.to_csv(project+'/matrix/gene_annotated.csv', sep='\t')
-    return
-#
-#
-def get_gene_expression(project):
-    gene_annotate = pandas.read_csv(project+'/matrix/gene_annotated.csv', sep='\t', index_col=0)
-    accesson_matrix = pandas.read_csv(project+'/matrix/Accesson_reads.csv', sep=',', index_col=0,
-                                      engine='c', na_filter=False, low_memory=False)
-    genes, matrix = [], []
-    for gene in gene_annotate.index.values:
-        accessons = gene_annotate.loc[gene, 'accessons'].split(';')
-        weight = list(map(float, gene_annotate.loc[gene, '-log10(P-value)'].split(';')))
-        if len(accessons)>0:
-            sub_matrix = accesson_matrix[accessons].values
-            expression = numpy.average(sub_matrix, axis=1, weights=weight).T
-            matrix.append(expression)
-            genes.append(gene)
-    matrix = numpy.array(matrix).T
-    expression_df = pandas.DataFrame(matrix, index=accesson_matrix.index, columns=genes)
-    expression_df.to_csv(project+'/matrix/gene_expression.csv', sep=',')
-    return
-#
-#
-def gene_expression(project, genome="hg19", width=1000000, pvalue=0.01):
-    """
-    project:     Path to the project folder.
-    genome:      Genome reference for Homer, can be "hg19" or "mm10", default="hg19".
-    width:       Width of Genome region for fisher exact test, default=1000000.
-    pvalue:      P-value threshold for fisher exact test, default=0.01.
-    """
-    annotate_peak(project, genome=genome)
-    annotate_accessons(project, width=width, pvalue=pvalue)
-    get_gene_expression(project)
-    print('gene_expression output files:')
-    print(project+'/matrix/Accesson_annotated.csv')
-    print(project+'/matrix/gene_annotated.csv')
-    print(project+'/matrix/gene_expression.csv')
-    return
-#
-#
 ######## generate motif-cell deviation matrix
 #
 #
@@ -352,8 +240,8 @@ def motif_search(info):
     motif_name = motif.split('-')[-1]
 #    print(motif_name)
     fimoFile = outFolder + '/' + motif +'.fimo'
-    os.popen('fimo --bgfile ' + bgFile + ' --text --thresh ' + threshold + ' --motif ' + motif_name
-        + ' --no-qvalue --verbosity 1 ' + motifFile + ' ' + motifFasta + ' > ' + fimoFile)
+    subprocess.check_call('fimo --bgfile ' + bgFile + ' --text --thresh ' + threshold + ' --motif ' + motif_name
+        + ' --no-qvalue --verbosity 1 ' + motifFile + ' ' + motifFasta + ' > ' + fimoFile, shell=True)
     bedFile = outFolder + '/' + motif +'.bed'
     with open(fimoFile) as fimo, open(bedFile, 'w') as bed:
         for line in fimo:
@@ -367,8 +255,8 @@ def motif_search(info):
                 strand, score, pvalue, name = words[4], words[5], words[6], words[0]
                 newLine = chrom+'\t'+str(start)+'\t'+str(end)+'\t'+strand+'\t'+score+'\t'+pvalue+'\t'+name
                 bed.write(newLine+'\n')
-    os.popen('gzip ' + bedFile)
-    os.popen('rm ' + fimoFile)
+    subprocess.check_call('gzip ' + bedFile, shell=True)
+    subprocess.check_call('rm ' + fimoFile, shell=True)
     return
 #
 #
@@ -412,7 +300,7 @@ def score_peaks(peaks_file, motif_folder, out_file):
             chip = {}
             for line in chipData:
                 chrom, start, end = line[0], int(line[1]), int(line[2])
-                if not chip.has_key(chrom):
+                if chrom not in chip.keys():
                     chip[chrom] = [[start, end]]
                 else:
                     chip[chrom].append([start, end])
@@ -454,7 +342,7 @@ def motif_matrix(project,
             words = line.split('\t')
             leave = words[0:3]
             temp02.write('\t'.join(leave)+'\n')
-    os.popen('bedtools nuc -fi ' + genome_fa + ' -bed ' + temp02_file + ' > ' + temp03_file)
+    subprocess.check_call('bedtools nuc -fi ' + genome_fa + ' -bed ' + temp02_file + ' > ' + temp03_file, shell=True)
     with open(temp03_file) as temp03, open(trans_bias, 'w') as bias:
         for i, line in enumerate(temp03):
             if i>0:
@@ -464,10 +352,10 @@ def motif_matrix(project,
 #
     motif_folder = project + '/matrix/motif'
     peaks_file = project + '/peak/top_filtered_peaks.bed'
-    if os.path.exists(motif_folder): os.popen('rm -rf ' + motif_folder)
-    os.popen('mkdir ' + motif_folder)
+    if os.path.exists(motif_folder): subprocess.check_call('rm -rf ' + motif_folder, shell=True)
+    subprocess.check_call('mkdir ' + motif_folder, shell=True)
     motifFasta = project + '/matrix/motif.fasta'
-    os.popen('bedtools getfasta -fi ' + genome_fa + ' -bed ' + peaks_file + ' -fo ' + motifFasta)
+    subprocess.check_call('bedtools getfasta -fi ' + genome_fa + ' -bed ' + peaks_file + ' -fo ' + motifFasta, shell=True)
     batch_fimo(background, pvalue, meme, motifFasta, motif_folder, np)
     TFmatrix_file = project + '/matrix/motif_filtered.csv'
     score_peaks(peaks_file, motif_folder, TFmatrix_file)
@@ -479,11 +367,11 @@ def motif_matrix(project,
 ######## generate differential features
 #
 #
-def group_cells(project, cell_label, cluster, target, vs):
+def group_cells(project, cell_label, target, vs):
     if cell_label=='notes':
         cells_csv = project+'/matrix/filtered_cells.csv'
     elif cell_label=='cluster':
-        cells_csv = project+'/result/'+cluster+'_cluster_by_APEC.csv'
+        cells_csv = project+'/result/cluster_by_APEC.csv'
     else:
         print("Error: wrong <cell_label>, should be 'notes' or 'cluster' !")
         sys.exit()
@@ -493,8 +381,8 @@ def group_cells(project, cell_label, cluster, target, vs):
     if 'cluster' not in cells_df.columns.values:
         cells_df['cluster'] = cells_df['notes']
     else:
-        kCluster = map(int, kCluster)
-        if vs!='all': vsCluster = map(int, vsCluster)
+        kCluster = list(map(int, kCluster))
+        if vs!='all': vsCluster = list(map(int, vsCluster))
     if vs=='all': vsCluster = list(set(cells_df['cluster'].values)-set(kCluster))
     cell_inCluster = cells_df.loc[cells_df['cluster'].isin(kCluster)].index.values
     cell_outCluster = cells_df.loc[cells_df['cluster'].isin(vsCluster)].index.values
@@ -505,9 +393,7 @@ def group_cells(project, cell_label, cluster, target, vs):
 #
 def differential_feature(project,
                          feature='accesson',   # ='accesson' or 'motif' or 'gene'
-                         gene_value='expression',
                          cell_label='cluster',   # ='cluster' or 'notes'
-                         cluster='louvain',   # ='louvain' or 'KNN'
                          target='1',   # ='1'
                          vs='all',   # ='all' or '2' or '2,3,4'
                          pvalue=0.01,
@@ -518,15 +404,10 @@ def differential_feature(project,
     feature:     Type of feature, can be 'accesson' or 'motif' or 'gene', default='accesson'.
                  If feature='accesson', run clustering.cluster_byAccesson() first;
                  if feature='motif', run clustering.cluster_byMotif() first;
-                 if feature='gene', run generate.gene_expression() first.
-    gene_value:  Using 'expression' or 'score' to search for differential genes. default='expression'.
-                 This parameter is only valid when feature='gene'.
-                 If gene_value='expression', run generate.gene_expression() first.
-                 If gene_value='score', run generate.gene_score() first.
+                 if feature='gene', run generate.gene_score() first.
     cell_label:  Cell labels used for differential analysis, can be 'notes' or 'cluster', default='cluster'.
-    cluster:     Clustering algorithm used in clustering.cluster_byXXX(), can be 'louvain' or 'KNN', default='louvain'.
     target:      The target cluster that users search for differential features, default='1'.
-                 if cell_label='cluster', target is one element in the 'cluster' column of XXX_cluster_by_XXX.csv file;
+                 if cell_label='cluster', target is one element in the 'cluster' column of cluster_by_APEC.csv;
                  if cell_label='notes', target is one element in the 'notes' column of filtered_cells.csv file.
     vs:          Versus which clusters, can be '2,3,4' or 'all', default='all' (means all the rest clusters).
     pvalue:      P-value for student-t test, default=0.01.
@@ -537,17 +418,17 @@ def differential_feature(project,
     elif feature=='motif':
         reads_file = project+'/result/deviation_chromVAR.csv'
     elif feature=='gene':
-        if gene_value=='expression':
-            reads_file = project+'/matrix/gene_expression.csv'
-        elif gene_value=='score':
-            reads_file = project+'/matrix/genes_scored_by_TSS_peaks.csv'
+#        if gene_value=='expression':
+#            reads_file = project+'/matrix/gene_expression.csv'
+#        elif gene_value=='score':
+        reads_file = project+'/matrix/genes_scored_by_TSS_peaks.csv'
     else:
         print("Error: wrong <feature>, should be 'accesson' or 'motif' or 'gene' !")
         sys.exit()
     reads_df = pandas.read_csv(reads_file, sep=',', index_col=0,
                                engine='c', na_filter=False, low_memory=False)
     if feature=='motif': reads_df = reads_df.T
-    cell_inCluster, cell_outCluster = group_cells(project, cell_label, cluster, target, vs)
+    cell_inCluster, cell_outCluster = group_cells(project, cell_label, target, vs)
     read_inCluster = reads_df.loc[cell_inCluster].values
     read_outCluster = reads_df.loc[cell_outCluster].values
     mean_in, mean_out = read_inCluster.mean(axis=0), read_outCluster.mean(axis=0)
@@ -564,17 +445,21 @@ def differential_feature(project,
     matrix_df = matrix_df.sort_values(by=['p-value'])
     nearby_csv = project+'/peak/peaks_nearby_genes.csv'
     if (feature=='accesson') & (os.path.exists(nearby_csv)):
-        peaks_df = pandas.read_csv(project+'/matrix/Accesson_peaks.csv', sep='\t')
-        nearby_genes_df = pandas.read_csv(nearby_csv, sep='\t')
+        peaks_df = pandas.read_csv(project+'/matrix/Accesson_peaks.csv', sep='\t', index_col=0)
+        nearby_genes_df = pandas.read_csv(nearby_csv, sep='\t', index_col=0)
         for accesson in matrix_df.index.values:
-            peaks = peaks_df.loc[peaks_df['group']==int(accesson)].index.values
-            genes = nearby_genes_df.loc[peaks, 'nearby_genes'].values
-            genes = [str(x) for x in genes if (x!='NONE')&(type(x)==str)]
-            genes = list(set(genes))
-            genes.sort()
-            genes = ';'.join(genes)
-            matrix_df.loc[accesson, 'relevant_genes'] = genes
-    out_csv = project+'/result/differential_'+feature+'_of_cluster_'+target+'_vs_'+'_'.join(vs.split(','))+'.csv'
+            peaks = [x for x in peaks_df.loc[peaks_df['group']==int(accesson)].index.values]
+            peaks = list(set(peaks).intersection(set(nearby_genes_df.index.values)))
+            if len(peaks)>0:
+                genes = nearby_genes_df.loc[peaks, 'nearby_genes'].values
+                genes = [str(x) for x in genes if (x!='NONE')&(type(x)==str)]
+                genes = list(set(genes))
+                genes.sort()
+                genes = ';'.join(genes)
+                matrix_df.loc[accesson, 'relevant_genes'] = genes
+            else:
+                matrix_df.loc[accesson, 'relevant_genes'] = 'NONE'
+    out_csv = project+'/result/differential_'+feature+'_of_cluster_'+'_'.join(target.split(','))+'_vs_'+'_'.join(vs.split(','))+'.csv'
     matrix_df.to_csv(out_csv, sep='\t')
     print('differential_feature output files:')
     print(out_csv)
@@ -587,7 +472,7 @@ def differential_feature(project,
 def peaks_of_accesson(project, accesson=1):
     """
     project:     Path to the project folder.
-    accesson:    Number of accesson that users want to get the peak list.
+    accesson:    Name/number of accesson that users want to get the peak list.
     """
     peaks_df = pandas.read_csv(project+'/matrix/Accesson_peaks.csv', sep='\t', index_col=0)
     peaks_bed = numpy.array(open(project+'/peak/top_filtered_peaks.bed').readlines())
@@ -606,56 +491,54 @@ def peaks_of_accesson(project, accesson=1):
 ######## generate potential super enchancer
 #
 #
-def search_super_enhancer(project, super_range=1000000):
+def super_enhancer(project, super_range=1000000, p_cutoff=0.01):
     """
     project:         Path to the project folder.
     super_range:    Genome range to search for super enhancer, default=1000000.
+    p_cutoff:       Cutoff of p-value, default=0.01.
     """
     peaks = numpy.array([x.split()[:3] for x in open(project+'/peak/top_filtered_peaks.bed').readlines()])
-    chroms = list(set(peaks[:, 0]))
-    chroms.sort()
+    peaks_df = pandas.DataFrame(peaks[:,0], index=['peak'+str(i) for i in range(0, len(peaks))], columns=['chrom'])
+    peaks_df['start'] = list(map(int, peaks[:,1]))
+    peaks_df['end'] = list(map(int, peaks[:,2]))
     accesson_df = pandas.read_csv(project+'/matrix/Accesson_peaks.csv', sep='\t', index_col=0)
     accessons = list(set(accesson_df['group'].values))
-    all_supers, all_locate, all_base = [], [], []
+    nearby_peaks = {}
+    for peak in peaks_df.index.values:
+        chrom, start, end = peaks_df.loc[peak, 'chrom'], peaks_df.loc[peak, 'start'], peaks_df.loc[peak, 'end']
+        position1, position2 = start-super_range, end+super_range
+        same_chrom_df = peaks_df.loc[peaks_df['chrom']==chrom]
+        in_range_df = same_chrom_df.loc[(same_chrom_df['end']>position1)&(same_chrom_df['start']<position2)]
+        nearby_peaks[peak] = in_range_df.index.values
+    all_supers = {}
     for access in accessons:
-        peaks_group = accesson_df.loc[accesson_df['group']==access].index.values
-        peaks_index = [int(x[4:]) for x in peaks_group]
-        peaks_info = peaks[peaks_index, :]
-        peaks_dict = {x:[] for x in chroms}
-        peaks_label = {x:[] for x in chroms}
-        for ip,pp in enumerate(peaks_info):
-            peaks_dict[pp[0]].append((int(pp[1])+int(pp[2]))/2)
-            peaks_label[pp[0]].append(peaks_index[ip])
-        peaks_label = {x:numpy.array(peaks_label[x]) for x in peaks_label.keys()}
-        supers, locate, base = [], [], []
-        for chrom in peaks_dict.keys():
-            if len(peaks_dict[chrom])>=2:
-                position = numpy.array(peaks_dict[chrom])
-                super = [numpy.where(abs(position-x)<=super_range)[0] for x in position if
-                      len(numpy.where(abs(position-x)<=super_range)[0])>1]
-                super = ['-'.join(map(str, x)) for x in super]
-                super = list(set(super))
-                super = [list(map(int, x.split('-'))) for x in super]
-                super_peaks = numpy.array(['-'.join(map(str, peaks_label[chrom][x])) for x in super])
-                if len(super_peaks)>0:
-                    for ii,ss in enumerate(super_peaks):
-                        peaks_in = map(int, ss.split('-'))
-                        start = peaks[peaks_in, 1:].astype(int).min()
-                        end = peaks[peaks_in, 1:].astype(int).max()
-                        delta = numpy.array([abs(peaks_in[i+1]-x) for i,x in enumerate(peaks_in[:-1])])
-                        close = numpy.where(delta<=2)[0]
-                        percent = len(close)/float(len(delta))
-                        if (len(delta)>=2) & (percent>0.5) :
-                            supers.append(ss)
-                            locate.append(access)
-                            base.append(chrom+':'+str(start)+'-'+str(end))
-        all_supers.extend(supers)
-        all_locate.extend(locate)
-        all_base.extend(base)
-    super_df = pandas.DataFrame(numpy.array([all_locate, all_base]).T, index=all_supers, columns=['peaks', 'position'])
-    super_df.to_csv(project+'/result/potential_super_enhancer.csv', sep='\t')
-    print('search_super_enhancer output files:')
-    print(project+'/result/potential_super_enhancer.csv')
+        acc_peaks = accesson_df.loc[accesson_df['group']==access].index.values
+        for peak in acc_peaks:
+            nearby = nearby_peaks[peak]
+            overlap = list(set(nearby).intersection(set(acc_peaks)))
+            chrom = peaks_df.loc[overlap[0], 'chrom']
+            start = peaks_df.loc[overlap, 'start'].min()
+            end = peaks_df.loc[overlap, 'end'].max()
+            position = chrom+':'+str(start)+'-'+str(end)
+            percent = len(overlap)/float(len(nearby))
+            overlap_peaks = '-'.join(overlap)
+            if overlap_peaks not in all_supers:
+                all_supers[overlap_peaks] = [position, percent, len(overlap)]
+            elif all_supers[overlap_peaks][1]<percent:
+                all_supers[overlap_peaks] = [position, percent, len(overlap)]
+    matrix_df = pandas.DataFrame.from_dict(all_supers, orient='index', columns=['position', 'percent', 'n_peaks_in_accesson'])
+    all_percents = matrix_df['percent']
+    print(type(all_percents[0]))
+    pvalues = []
+    for percent in all_percents:
+        higher = numpy.where(all_percents>=percent)[0]
+        pvalues.append(len(higher)/float(len(all_percents)))
+    matrix_df['p-value'] = pvalues
+    matrix_df = matrix_df.sort_values(by=['p-value'])
+    matrix_df = matrix_df.loc[matrix_df['p-value']<p_cutoff]
+    matrix_df = matrix_df.loc[matrix_df['n_peaks_in_accesson']>=5]
+    matrix_df.index = [str(x) for x in range(1, len(matrix_df.index)+1)]
+    matrix_df.to_csv(project+'/result/potential_super_enhancer.csv', sep='\t')
     return
 #
 #
